require 'mapdata_storage'
require 'mapdata_dbc'
require 'erb'

class Loader
  def Loader.load_wma(path)
      loadfile = WorldMapAreaFile.new(path)
      hash = Hash.new
      loadfile.each do |r|
        var = WorldMapArea.create(r)
        if var.wma_id == 542 
          next
        end
        if var.map_id == 720
          var.dungeon_map_id = -1
        end
        hash[var.wma_id] = var
      end
      return hash
  end
  def Loader.load_dm(path)
    loadfile = DungeonMapFile.new(path)
    hash = Hash.new
    loadfile.each do |r|
      var = DungeonMap.create(r)
      # Skip floors defined as 0,0 for w,h
      if var.width == 0 and var.height == 0
        next
      end
      hash[var.sub_id] = var
    end
    return hash
  end
  def Loader.load_wmc(path)
    loadfile = WorldMapContinentFile.new(path)
    hash = Hash.new
    loadfile.each do |r|
      var = WorldMapContinent.create(r)
      hash[var.wmc_id] = var
      #puts var.inspect
    end
    return hash
  end
  
  def Loader.load_wmaps(path)
    loadfile = MapFile.new(path)
    hash = Hash.new
    loadfile.each do |r|
      var = Map.create(r)
      hash[var.map_id] = var
    end
    return hash    
  end
  
  def Loader.load_transforms(path)
    loadfile = WorldMapTransformFile.new(path)
    hash = Hash.new
    loadfile.each do |r|
      var = WorldMapTransforms.create(r)
      hash[var.map_id] = var
    end
    return hash    
  end
  
  def Loader.add_floors_to_maps(maps,floors)
      lmap = maps.sort
      lmap.each do |map_block|
        map_key = map_block[0]
        map_data = maps[map_key]
        inc_floors = false
        # Dragonsoul hack
        if (map_data.flags == 260 || map_data.area_id == 6738) and map_data.dungeon_map_id > -1
          map_data.map_floors << map_data.to_floor
          inc_floors = true
        end
        if map_data.flags == 16
          puts "Skipping #{map_data.name} floors for #{map_key} with flags #{map_data.flags}"
          next
        end
        floors.each_pair do |fkey,floor_info|
          if floor_info.wma_id == map_data.map_id
            if floor_info.is_continent?
              # Dalaran special case
              if floor_info.wma_id == 571 && map_data.area_id == 4395
                # We need a little special handling here for dalaran
                puts "Adding floor #{floor_info.floor}"
                map_data.map_floors << floor_info
                floors.delete(fkey)
              end 
            else
              if inc_floors
                puts "First floor hack for #{map_data.name}"
                floor_info.floor += 1
              end
              puts "Adding floor #{floor_info.inspect}"
              map_data.map_floors << floor_info
              floors.delete(fkey)
            end
          end
        end
        puts "Map #{map_data.name} (#{map_data.dungeon_map_id}|#{map_data.area_id}|#{map_data.wma_id}|#{map_data.relate_map_id}|#{map_data.virtual_id} ) Flags #{map_data.flags} Floors #{map_data.map_floors.size}"
        map_data.map_floors.delete_if do |a| a.instance_of?(String) end
        if map_data.map_floors.length > 0
          map_data.map_floors.sort!
          map_data.map_floors.uniq!
        end
        if map_data.map_floors.length == 1 && map_data.map_floors[0].width == map_data.width && map_data.map_floors[0].height == map_data.height
          map_data.map_floors = []
        end
        # pass remaing floor data || floor_info.wmc_id == map_data.wma_id
        maps[map_key] = map_data
        #if map_data.map_floors.length == 0
        #  map_data.map_floors << map_data.to_floor
        #end
        floors.each_pair do |fkey,floor_info|
          if floor_info.wmc_id == map_data.wma_id && floor_info.wma_id == map_data.relate_map_id
            map_data.micro << floor_info
          end
        end
      end
      
  end
  
  def Loader.load_all(path)
    world_map_area_entries = load_wma(path)
    dungeon_map_floors = load_dm(path)
    world_map_cntinents = load_wmc(path)
    world_map_transforms = load_transforms(path)
    add_floors_to_maps(world_map_area_entries,dungeon_map_floors)
    # Speicla handling for argetn colesium
    acM = world_map_area_entries[543]
    ncM = WorldMapArea.new
    ncM = acM.clone
    ncM.wma_id = 542
    ncM.map_id = acM.map_id + 1
    ncM.map_floors = []
    ncM.map_floors << acM.map_floors[0].clone
    world_map_area_entries[542] = ncM
    # End argent colesium hack
    # Firelands hack
    #firelands = world_map_area_entries[800]
    #firelands.map_floors[0].floor = 1
    #firelands.map_floors[1].floor = 2
    fullmaps = load_wmaps(path)
    world_map_area_entries.each_key do |wmk|
      data = world_map_area_entries[wmk]
      unless fullmaps[data.map_id].nil?
        data.map_type = fullmaps[data.map_id].map_type
        data.instance_continent = fullmaps[data.map_id].entrace_map_id
      end
      world_map_area_entries[wmk] = data
    end
    return world_map_area_entries
  end
  
end

path = ARGV[0]
data = Loader.load_all(path)

output = ARGV[1]
output ||= "LibMapData.lua"

build = ARGV[2]
build ||= 999

# Load ERB template, and process it
 erb_file = IO.read("lib_template.erb")
 template = ERB.new(erb_file,nil, '%><>')
 results = template.result
 fh = File.new("#{output}", "w")
 fh.write(results)
 fh.close